<?php
/**

Copyright (c) 2009, SilverStripe Australia PTY LTD - www.silverstripe.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the 
      documentation and/or other materials provided with the distribution.
    * Neither the name of SilverStripe nor the names of its contributors may be used to endorse or promote products derived from this software 
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
 
*/

/**
 * A page type that provides wiki like editing functionality. The  
 * initial goals are
 * 
 * - automatically create child pages based on [[page name]] style syntax
 * - be able to edit content via a wysiwyg mechanism
 * - add links by picking pages from the tree selection control
 * - add images picking images from the tree selection control
 * 
 * @author Marcus Nyeholt <marcus@silverstripe.com.au>
 *
 */
class WikiPage extends Page
{
	public static $db = array();

	protected function onBeforeWrite()
	{
		parent::onBeforeWrite();
		
		$newPages = $this->parseNewPagesFrom($this->Content);
		foreach ($newPages as $pageTitle) {
			$trimmedTitle = trim($pageTitle);
			
			// need to trim the title before doing the search
			$page = DataObject::get_one('WikiPage', "SiteTree.Title='".Convert::raw2sql($trimmedTitle)."'");

			if (!$page) {
				// it's a new page, so create that
				$page = new WikiPage();
				$page->Title = $trimmedTitle;
				$page->MenuTitle = $trimmedTitle;
				$page->ParentID = $this->ID;
				$page->write();
			}

			$replacement = '<a href="[sitetree_link id='.$page->ID.']">'.$pageTitle.'</a>';
			$this->Content = str_replace('[['.$pageTitle.']]', $replacement, $this->Content);
		}
	}
	
	/**
	 * Separated into a separate method for testing
	 * 
	 * @param String $content
	 * @return array
	 */
	public function parseNewPagesFrom($content)
	{
		$pages = array();
		
		if (preg_match_all('/\[\[([\w\s_.-]+)\]\]/', $content, $matches)) {
			// exit(print_r($matches));
			foreach ($matches[1] as $pageTitle) {
				$pages[] = $pageTitle;
			}
		}

		return $pages;
	} 
}

class WikiPage_Controller extends Page_Controller
{
	static $allowed_actions = array(
		'linkselector',
		'edit',
		'StatusForm',
		'save',
		'done',
		'publish',
		'startediting',
		'EditForm',
		'LinkSelectForm',
		'objectdetails',
	);
	
	/**
	 * We don't want extended wiki page functionality to be infested with
	 * the themes as defined in PageController's init function
	 * 
	 * @see mysite/code/pages/Page_Controller#init()
	 
	public function init() {
		ContentController::init();
	}*/

	/**
	 * The form we're editing with
	 * 
	 * @var Form
	 */
	protected $form;

	/**
	 * Action handler for editing this wiki page
	 * 
	 * Creates a form that's used for editing the page's content, 
	 * as well as adding in a couple of additional toolbar actions
	 * for adding a simple link and a simple image
	 */
	public function edit()
	{
		HtmlEditorField::include_js();
		Requirements::javascript('simplewiki/javascript/sslinks/editor_plugin_src.js');

		if(!$this->owner->canEdit()) {
			return Security::permissionFailure($this);
		}

		$this->form = $this->EditForm();

		// check who's editing and whether or not we should bail out
		return $this->renderWith(array('WikiPage', 'Page'));
	}

	/**
	 * Creates the form used for editing the page's content
	 * 
	 * @return Form
	 */
	public function EditForm()
	{
		// make sure to use content from staging... is this the correct way?
		Versioned::reading_stage('Stage');
		
		$record = DataObject::get_by_id('WikiPage', $this->owner->ID);

		$fields = new FieldSet(
			new HtmlEditorField('Content', '', 30, 20, $record->Content)
		);
		$actions = new FieldSet(
			new FormAction('save', _t('WikiPage.SAVE','Save')),
			new FormAction('done', _t('WikiPage.DONE','Done (Draft)')),
			new FormAction('publish', _t('WikiPage.PUBLISH','Publish'))
		);

		return new Form($this, "EditForm", $fields, $actions);
	}
	
	/**
	 * Save the submitted data
	 * 
	 * @return 
	 */
	public function save($data, $form)
	{
		// save stuff then reuse the edit action
		$form->saveInto($this->owner);
		$this->owner->write('Stage');

		Director::redirect($this->owner->Link('edit'));
	}

	/**
	 * Complete editing and publish the data
	 * 
	 * @param mixed $data
	 * @param Form $form
	 */
	public function done($data, $form)
	{
		// save stuff then reuse the edit action
		$form->saveInto($this->owner);
		$this->owner->writeToStage('Stage');

		Director::redirect($this->owner->Link().'?stage=Stage');
	}
	
	/**
	 * Complete editing and publish the data
	 * 
	 * @param mixed $data
	 * @param Form $form
	 */
	public function publish($data, $form)
	{
		// save stuff then reuse the edit action
		$form->saveInto($this->owner);
		$this->owner->writeToStage('Stage');
		$this->owner->doPublish();
		
		// Make sure we're on the live content now
		Versioned::reading_stage('Live');
		
		// and go 
		Director::redirect($this->owner->Link().'?stage=Live');
	}

	/**
	 * We only want to output content if we're not in edit mode
	 * at all
	 * 
	 * @return String
	 */
	public function Content()
	{
		if ($this->form) {
			return '';
		}

		return $this->owner->Content; //XML_val('Content');
	}

	/**
	 * Return the form to the user if it exists, otherwise some information
	 * about who is currently editing
	 * 
	 * @return Form
	 */
	public function Form()
	{
		if (!$this->form && $this->owner->canEdit()) {
			// create the information form 
			$this->form = $this->StatusForm();
		}

		return $this->form;
	}


	/**
	 * Gets the status form that is used by users to trigger the editing mode
	 * if they have the relevant access to it. 
	 * 
	 * @return Form
	 */
	public function StatusForm()
	{
		// TODO: Add a field to let the user know there's a draft that hasn't
		// been published yet
		// TODO: Add check to see if someone else is editing this page
		$fields = new FieldSet();
		$actions = new FieldSet(
			new FormAction('startediting', _t('WikiPage.STARTEDIT','Edit Page'))
		);
		
		return new Form($this, 'StatusForm', $fields, $actions);
	}

	/**
	 * Called to start editing this page
	 * 
	 */
	public function startediting()
	{
		Director::redirect($this->owner->Link('edit'));
	}
	
	/**
	 * Show the link selector
	 * 
	 * @return String
	 */
	public function linkselector()
	{
		return $this->renderWith(array('LinkSelectDialog'));
	}
	
	public function LinkingType()
	{
		return isset($_GET['type']) ? $_GET['type'] : 'href';
	}

	/**
	 * 
	 * @return Form
	 */
	public function LinkSelectForm()
	{
		$type = isset($_GET['type']) ? $_GET['type'] : 'href';

		$fields = new FieldSet(
			new TreeDropdownField('TargetPage', _t('WikiPage.TARGETPAGE', 'Select Page'), 'SiteTree'),
			new TreeDropdownField('TargetFile', _t('WikiPage.TARGETIMAGE', 'Select Image'), 'File')
		);	
		
		$actions = new FieldSet(
			new FormAction('insert', _t('WikiPage.INSERTLINK','Insert'))
		);

		return new Form($this, 'LinkSelectForm', $fields, $actions);
	}
	
	/**
	 * Retrieves information about a selected image
	 * 
	 * @return string
	 */
	public function objectdetails()
	{
		$response = new stdClass;
		if (isset($_GET['ID'])) {
			$type = null;
			if (isset($_GET['type'])) {
				$type = $_GET['type'] == 'href' ? 'SiteTree' : 'File';	
			} else {
				$type = 'SiteTree';
			}

			$object = DataObject::get_by_id($type, $_GET['ID']);
			
			$response->Title = $object->Title;
			$response->Link = $object->Link();

			if ($object instanceof Image) {
				$response->Name = $object->Name;
				$response->Filename = $object->Filename;
				$response->width = $object->getWidth();
				$response->height = $object->getHeight();
			}


			$response->error = 0;
		} else {
			$response->error = 1;
			$response->message = "Invalid image ID";
		}
		echo json_encode($response);
	}
}

/** 
 * Custom class for custom templating 
 * 
 * @author Marcus Nyeholt <marcus@silverstripe.com.au>
 *
 */
class LinkSelectForm extends Form {}

?>